note
   description: "Concrete Subject for storing and maintaining the time of day.%
   % It notifies its observers every second."

class CLOCK_TIMER
  inherit SUBJECT
		  rename make as make_from_subject end

create make

---------------------------
feature -- Creation

make (initial_time, tick_int : INTEGER)
  -- Creation and initialization with a time offset (in seconds)
  require
    positive_initial_time: initial_time >= 0
  do
    make_from_subject
    time := initial_time
    tick_interval := tick_int
  ensure
    time_initialized: initial_time\\86400 = 3600*hour + 60*minute + second
    tick_interval_set: tick_interval = tick_int
  end

---------------------------
feature {ANY} -- Queries

hour : INTEGER
  -- the current hour
  do
    Result := (time//3600)\\24
  end

minute : INTEGER
  -- the current minute
  do
    Result := (time//60)\\60
  end

second : INTEGER
  -- the current second
  do
    Result := time\\60
  end -- second

---------------------------
feature {ANY} -- Commands

tick
  -- Called by the internal timer to provide an accurate time base
  -- Call `notify' to inform observers of the change
  do
    time := time + tick_interval
    notify
  end

---------------------------
feature {NONE} -- Private

time : INTEGER                -- Elapsed time since initialization, in seconds
tick_interval : INTEGER       -- Number of seconds between ticks

invariant
  correct_time: time\\86400 = 3600*hour + 60*minute + second

end
